# RandomizeBookStack.py (Prototype 0.00)
# Test script for adding Ajax Easypose style ERC to a figure.

# 26k: Changed xTran, zTran, and yRotate to 'Nudge sideways', 'Nudge lengthwise', and 'Rotate' to reflect CR2 26k changes
# 26i: Changed 'Scale and 'yScale' to 'Book size' and 'Book thickness' to reflect CR2 test26i
#      Added re-hiding of all morph dials, because Poser11 automatically unhides them if altgeom switches
# 26g: Changed 'Cover height-width ratio' to 'Cover proportions' to match CR2 version 26g
# 0.01: Corrected yRot to use yRot min/max, not Twist !
#       Add option to use reduced translations range 
#       Removed xScale and zScale, added 'Cover height-width ratio' instead (to match BookStack4 test26f)
#       Added (very basic) setting of 'BOOK_0_TO_31' in the cover material for all nested stacks so they don't repeat books
# 0.00: Original, upl;oaded to post #94 here https://community.hivewire3d.com/threads/my-rather-infrequent-and-possibly-apocryphal-books-project.4492/page-5#post-232973


import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigure(self, thisfig):
		
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:15] == "Four Book Stack":
					self.figlist.append(figname)
					self.getAnyChildStackFigure(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:15] == "Four Book Stack"):
			poser.DialogSimple.MessageBox("This script is only for use with 'Four Book Stack' figures.\n\nScript aborted, no changes made.")
			return
		
		self.figlist.append(fig.Name())
		
		self.getAnyChildStackFigure(fig)
			
		bookcount = 0.0
		for f in self.figlist:
			fig = poser.Scene().Figure(f)
			st = fig.Material("BookMap64Cover").ShaderTree()
			nodelist = st.Nodes()
			for n in nodelist:
				if n.Name() == "BOOK_0_TO_31":
					n.InputByInternalName("Value_1").SetFloat(bookcount)
					bookcount = bookcount + 4
			# For each of the four books in the figure...
			i = 1
			while i < 5:
				# Get the next book
				bone = "book_" + str(i)
				act = fig.ActorByInternalName(bone)
								
				# Randomize geometry
				pname = "Book" + str(i)+"Geom"
				p = act.Parameter(pname)
				rand = random.randint(p.MinValue(),p.MaxValue()) #int, can be negative
				p.SetValue(rand)
			
				# Randomize 'Splay'
				p = act.Parameter("Splay")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Oblique'
				p = act.Parameter("Oblique")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Shear'
				p = act.Parameter("Shear")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Plump' - OVERRIDE -1 MINIMUM VALUE, USE 0
				p = act.Parameter("Plump")
				rand = random.uniform(0,p.MaxValue()) #float, can be negative
				p.SetValue(rand)
				
				# Randomize 'Shear'
				p = act.Parameter("Shear")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Twist'
				p = act.Parameter("Twist")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Curl (Paperback only)' - BEST NOT TO DO THIS VIA SCRIPT
				#p = act.Parameter("Curl (Paperback only)")
				#rand = random.uniform(p.MinValue(),p.MaxValue())
				#p.SetValue(rand)
			
				# Randomize rotation
				pmin=-10
				pmax=10
				p = act.Parameter("Rotate")
				# randomly select whether -10 to +10 or 170 to 190
				rand = random.uniform(-1,1)
				if rand > 0:
					pmin=170
					pmax=190
				rand = random.uniform(pmin,pmax)
				p.SetValue(rand)
			
				# Randomize 'xTran'
				pmin=0
				pmax=0
				p = act.Parameter("Nudge sideways")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'zTran'
				pmin=0
				pmax=0
				p = act.Parameter("Nudge lengthwise")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
			
				# Randomize 'Cover proportions'
				pmin=0
				pmax=0
				p = act.Parameter("Cover proportions")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'Scale'
				pmin=0
				pmax=0
				p = act.Parameter("Book size")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Randomize 'yScale'
				pmin=0
				pmax=0
				p = act.Parameter("Book thickness")
				rand = random.uniform(p.MinValue(),p.MaxValue())
				p.SetValue(rand)
				
				# Next book if any
				i = i + 1
			
		# Redraw what we've modified	
		poser.Scene().Draw()
	
		
		# If the geometry is changed then Poser11 unhides all the morphs for all geometries
		# So try to re-hide them
		for f in self.figlist:
			fig = poser.Scene().Figure(f)
			# For each of the four books in the figure...
			i = 1
			while i < 5:
				# Get the next book
				bone = "book_" + str(i)
				act = fig.ActorByInternalName(bone)
				nodename = "MT HBc" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Curl"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				
				i = i + 1
			
#poser.DialogSimple.MessageBox("Running 3DCheapskates RandomizeBookStack.py script (prototype 0.00)\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
#poser.DialogSimple.MessageBox("3DCheapskates RandomizeBookStack.py script (prototype 0.00) all done.")